/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */


package com.LibreGeoSocialApp.Utils;

import java.text.DecimalFormat;

import android.location.Location;


public class LocationUtils {

	/**
	 * The average tall (in meters) of a human being
	 * There is an approximated maximum error of +/- 0.25 meters
	 */
	public static final float MEAN_TALL = 1.75f;
	
	public static final int AUTO 		= 0;
	public static final int KILOMETERS 	= 1;
	public static final int METERS 	 	= 2;
	
	/**
	 * Earth's equatorial radius
	 */
	private static final float a = 6378137; 
	
	/**
	 * Earth's polar radius
	 */
	private static final float b = 6356752.3f; 
	
	/**
	 * The equivalent in meters for a distance of one latitude second
	 */
	private static final float LATITUDE_DEGREE = 30.82f;
	
    /**
     * Calculate the distance (in meters) between two Location points
     * 
     * @param Source Location Point
     * @param Destination Location Point
     * 
     * @return The distance in meters.
     */
	
	public static float calculateDistance (Location pointSource, Location pointDest)
	{
        
        float dist = pointSource.distanceTo(pointDest);
        
        return dist;
	}
	
	
    /**
     * Calculate the distance (in meters) between two Location points
     * 
     * @param The distance in meters
     * @param The kind of conversion (AUTO,KILOMETERS,METERS)
     * 
     * @return The String with the information of distance
     */
	
	public static String displayDistance (float dist, int unit)
	{
		
		String distString = null;
		
		switch (unit)
		{
			case AUTO:
				
				if (dist >= 1000.0)
					distString = distanceToKilometers(dist);
				else
					distString = distanceToMeters(dist);
				
				break;
			case KILOMETERS:
				
				distString = distanceToKilometers(dist);
				break;
				
			case METERS:
				
				distString = distanceToMeters(dist);
				break;
			
		}
		
		return distString;
		
		
	}
	
	private static String distanceToKilometers (float dist)
	{
		dist = (float) (dist / 1000.0);
		DecimalFormat df = new DecimalFormat("0.00");
		
		return String.valueOf(df.format(dist)) + " km.";
	}
	
	private static String distanceToMeters (float dist)
	{
		return String.valueOf(Math.round(dist)) + " m.";
	}
	
	/**
	 * This function estimates the roll the user has to use in order to see the resource
	 * @param distance is the distance to the resource
	 * @param ob_height is the height of the object
	 * @return This returns the roll in degrees
	 */
	public static float calculateRoll (float distance, float ob_height){
		float roll = 0, h;
		
		h = MEAN_TALL - ob_height;
		
		roll = (float) Math.toDegrees(Math.atan2(distance, h));
		
		return roll;
	}
	
	public static float calculateRollFromUser(float user_roll, float res_roll){
		float roll;
		
		roll = user_roll - res_roll;
		
		return roll;
	}
	
	/**
	 * This function calculates the azimuth of a resource from the user's position
	 * @param source is the user's (latitude, longitude)
	 * @param resource is the resource's (latitude, longitude)
	 * 
	 * @return This returns the azimuth in degrees
	 */
	public static float calculateResourceAzimuth(float[] source, float[] resource){
		float azimuth, dist_lat, dist_lng;
		
		float lat_degree = LATITUDE_DEGREE * 3600;
		float long_degree = (float) ((Math.PI/180)*Math.cos(Math.toRadians(source[0])) *
				Math.sqrt((Math.pow(a, 4)*Math.pow(Math.cos(Math.toRadians(source[0])),2) +
						Math.pow(b, 4)*Math.pow(Math.sin(Math.toRadians(source[0])),2))/
						(Math.pow(a, 2)*Math.pow(Math.cos(Math.toRadians(source[0])),2) +
								Math.pow(b, 2)*Math.pow(Math.sin(Math.toRadians(source[0])),2))
						));
		
		dist_lat = (resource[0] - source[0]) * lat_degree;
		dist_lng = (resource[1] - source[1]) * long_degree;
		
		azimuth = (float) Math.toDegrees(Math.atan2(dist_lng, dist_lat));
		
		if(azimuth<0)
			azimuth = 360+azimuth;
		
		return azimuth;
	}
	
	/**
	 * This function calculates the relative azimuth of the resource from the user
	 * @param user_azimuth is the azimuth of the user
	 * @param resource_azimuth is the azimuth of the resource
	 * @return The returned value is the angle in degrees
	 */
	public static float calculateAzimuthFromUser(float user_azimuth, float resource_azimuth){
		float azimuth = 500, azimuth_Aux;
		
		if(Math.abs(azimuth_Aux =  resource_azimuth - user_azimuth) < Math.abs(azimuth))
			azimuth = azimuth_Aux;
		
		if(Math.abs(azimuth_Aux =  (resource_azimuth - 360) - user_azimuth) < Math.abs(azimuth))
			azimuth = azimuth_Aux;
		
		if(Math.abs(azimuth_Aux =  (360 + resource_azimuth) - user_azimuth) < Math.abs(azimuth))
			azimuth = azimuth_Aux;
		
		
		return azimuth;
	}
	
}
